package net.wanji.opt.kafka;

import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.dto.CrossSchedulesDTO;
import net.wanji.opt.dto.GreenBeltKafkaDTO;
import net.wanji.opt.vo.GreenBeltInfoVO;
import org.apache.kafka.clients.consumer.Consumer;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.listener.KafkaListenerErrorHandler;
import org.springframework.kafka.listener.ListenerExecutionFailedException;
import org.springframework.messaging.Message;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author duanruiming
 * @date 2024/11/18 23:40
 * @description 消费神思推送kafka数据，不同kafka
 */
@Component
@Slf4j
public class KafkaListeners implements KafkaListenerErrorHandler  {

   @KafkaListener(topics = {"green_belt_info_data"}, groupId = "group2", containerFactory = "kafkaListenerContainerFactory2")
    public void receiveGreenBeltInfoData(String message) throws Exception {
        if (StringUtils.isNotBlank(message)) {
            message = greenBeltInfoData;
            ObjectMapper mapper = JacksonUtils.getInstance();
            GreenBeltKafkaDTO greenBeltKafkaDTO = mapper.readValue(greenBeltInfoData, GreenBeltKafkaDTO.class);
        }
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException e) {
        return new Object();
    }

    @Override
    @NonNull
    public Object handleError(Message<?> message, ListenerExecutionFailedException exception, Consumer<?, ?> consumer) {
        String errorMessage = String.format("监听主题：%s，消费者详情：%s，异常信息：%s，消息详情：%s",
                consumer.listTopics(), consumer.groupMetadata(), exception, message);
        log.error(errorMessage);
        return KafkaListenerErrorHandler.super.handleError(message, exception, consumer);
    }

    public static void main(String[] args) throws Exception {
        String message = greenBeltInfoData;
        ObjectMapper mapper = JacksonUtils.getInstance();
        GreenBeltKafkaDTO kafkaDTO = mapper.readValue(message, GreenBeltKafkaDTO.class);
        // 正反绿波容器
        List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails = new ArrayList<>();
        // 构建正向绿波
        setForward(kafkaDTO, dirGreenDetails);
        // 反向绿波
        setBackward(kafkaDTO, dirGreenDetails);
        // 返回绿波实体
        GreenBeltInfoVO greenBeltInfoVO = new GreenBeltInfoVO();
        greenBeltInfoVO.setGreenId(kafkaDTO.getGreenbeltId());
        greenBeltInfoVO.setLength(kafkaDTO.getGreenbeltLength());
        greenBeltInfoVO.setCycle(kafkaDTO.getMaxCycle());
        greenBeltInfoVO.setControlTime(kafkaDTO.getControlTime());
        greenBeltInfoVO.setControlDuration(kafkaDTO.getControlDuration());
        greenBeltInfoVO.setType(kafkaDTO.getGreenbeltType());
        greenBeltInfoVO.setDynamic(kafkaDTO.getDynamic());
        greenBeltInfoVO.setControlMethod(kafkaDTO.getControlMethod());
        greenBeltInfoVO.setDirGreenDetails(dirGreenDetails);
        System.err.println(mapper.writeValueAsString(greenBeltInfoVO));
    }

    private static void setBackward(GreenBeltKafkaDTO kafkaDTO, List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails) {
        // 反向绿波路口列表
        List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = new ArrayList<>();
        Map<String, Double> phaseStartBackward = kafkaDTO.getPhaseStartBackward();
        for (Map.Entry<String, Double> backEntry : phaseStartBackward.entrySet()) {
            String crossId = backEntry.getKey();
            Double phaseStartTime = backEntry.getValue();
            GreenBeltInfoVO.CrossGreenDetail crossGreenDetail = new GreenBeltInfoVO.CrossGreenDetail();
            crossGreenDetail.setCrossId(crossId);
            crossGreenDetail.setPhaseStartTime(phaseStartTime);
            Double phaseEndTime = kafkaDTO.getPhaseEndBackward().get(crossId);
            crossGreenDetail.setPhaseEndTime(phaseEndTime);
            Double greenStartTime = kafkaDTO.getBeltStartBackward().get(crossId);
            crossGreenDetail.setGreenStartTime(greenStartTime);
            Double speed = kafkaDTO.getSpeedBackward().get(crossId);
            crossGreenDetail.setSpeed(speed);
            Double offset = kafkaDTO.getOffset().get(crossId);
            crossGreenDetail.setOffset(offset);
            crossGreenDetails.add(crossGreenDetail);
        }
        // 反向绿波
        // 反向绿波实体
        GreenBeltInfoVO.DirGreenDetail dirGreenDetail = new GreenBeltInfoVO.DirGreenDetail();
        dirGreenDetail.setDirType(0);
        String forwardDirection = kafkaDTO.getBackwardDirection();
        dirGreenDetail.setDir(forwardDirection);
        double maxSpeedForward = kafkaDTO.getMaxSpeedBackward();
        dirGreenDetail.setMaxSpeed(maxSpeedForward);
        double minSpeedForward = kafkaDTO.getMinSpeedBackward();
        dirGreenDetail.setMinSpeed(minSpeedForward);
        double greenWidthForward = kafkaDTO.getGreenWidthBackward();
        dirGreenDetail.setGreenWithTime(greenWidthForward);
        dirGreenDetail.setCrossGreenDetailList(crossGreenDetails);
        dirGreenDetails.add(dirGreenDetail);
    }

    private static void setForward(GreenBeltKafkaDTO kafkaDTO, List<GreenBeltInfoVO.DirGreenDetail> dirGreenDetails) {
        // 正向绿波路口列表
        List<GreenBeltInfoVO.CrossGreenDetail> crossGreenDetails = new ArrayList<>();
        Map<String, Double> phaseStartForward = kafkaDTO.getPhaseStartForward();
        for (Map.Entry<String, Double> forwardEntry : phaseStartForward.entrySet()) {
            String crossId = forwardEntry.getKey();
            Double phaseStartTime = forwardEntry.getValue();
            GreenBeltInfoVO.CrossGreenDetail crossGreenDetail = new GreenBeltInfoVO.CrossGreenDetail();
            crossGreenDetail.setCrossId(crossId);
            crossGreenDetail.setPhaseStartTime(phaseStartTime);
            Double phaseEndTime = kafkaDTO.getPhaseEndForward().get(crossId);
            crossGreenDetail.setPhaseEndTime(phaseEndTime);
            Double greenStartTime = kafkaDTO.getBeltStartForward().get(crossId);
            crossGreenDetail.setGreenStartTime(greenStartTime);
            Double speed = kafkaDTO.getSpeedForward().get(crossId);
            crossGreenDetail.setSpeed(speed);
            Double offset = kafkaDTO.getOffset().get(crossId);
            crossGreenDetail.setOffset(offset);
            crossGreenDetails.add(crossGreenDetail);
        }
        // 正向绿波
        // 正向绿波实体
        GreenBeltInfoVO.DirGreenDetail dirGreenDetail = new GreenBeltInfoVO.DirGreenDetail();
        dirGreenDetail.setDirType(1);
        String forwardDirection = kafkaDTO.getForwardDirection();
        dirGreenDetail.setDir(forwardDirection);
        double maxSpeedForward = kafkaDTO.getMaxSpeedForward();
        dirGreenDetail.setMaxSpeed(maxSpeedForward);
        double minSpeedForward = kafkaDTO.getMinSpeedForward();
        dirGreenDetail.setMinSpeed(minSpeedForward);
        double greenWidthForward = kafkaDTO.getGreenWidthForward();
        dirGreenDetail.setGreenWithTime(greenWidthForward);
        dirGreenDetail.setCrossGreenDetailList(crossGreenDetails);
        dirGreenDetails.add(dirGreenDetail);
    }

    public static String greenBeltInfoData = "{\n" +
            "\t\"backward_direction\": \"e2w\",\n" +
            "\t\"belt_start_backward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 0.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 87.0,\n" +
            "\t\t\"13MS20B5SI0\": 7.0,\n" +
            "\t\t\"13MUK0B5SH0\": 79.0,\n" +
            "\t\t\"13N0F0B5SH0\": 3.0,\n" +
            "\t\t\"13N200B5SH0\": 86.0\n" +
            "\t},\n" +
            "\t\"belt_start_forward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 0.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 87.0,\n" +
            "\t\t\"13MS20B5SI0\": 7.0,\n" +
            "\t\t\"13MUK0B5SH0\": 79.0,\n" +
            "\t\t\"13N0F0B5SH0\": 3.0,\n" +
            "\t\t\"13N200B5SH0\": 86.0\n" +
            "\t},\n" +
            "\t\"control_duration\": 90,\n" +
            "\t\"control_method\": 1,\n" +
            "\t\"control_time\": \"2024-11-18T18:39:55\",\n" +
            "\t\"dynamic\": 1,\n" +
            "\t\"forward_direction\": \"w2e\",\n" +
            "\t\"green_width_backward\": 35.9999999999999,\n" +
            "\t\"green_width_forward\": 80.0,\n" +
            "\t\"greenbelt_direction\": 0,\n" +
            "\t\"greenbelt_id\": \"road1\",\n" +
            "\t\"greenbelt_length\": 2757.0,\n" +
            "\t\"greenbelt_type\": 2,\n" +
            "\t\"max_cycle\": 220,\n" +
            "\t\"max_speed_backward\": 45.0,\n" +
            "\t\"max_speed_forward\": 35.0,\n" +
            "\t\"min_speed_backward\": 35.0,\n" +
            "\t\"min_speed_forward\": 25.0,\n" +
            "\t\"offset\": {\n" +
            "\t\t\"13MOD0B5SI0\": 198.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 46.0,\n" +
            "\t\t\"13MS20B5SI0\": 89.0,\n" +
            "\t\t\"13MUK0B5SH0\": 216.0,\n" +
            "\t\t\"13N0F0B5SH0\": 201.0,\n" +
            "\t\t\"13N200B5SH0\": 89.0\n" +
            "\t},\n" +
            "\t\"phase_end_backward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 132.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 165.0,\n" +
            "\t\t\"13MS20B5SI0\": 148.0,\n" +
            "\t\t\"13MUK0B5SH0\": 115.0,\n" +
            "\t\t\"13N0F0B5SH0\": 165.0,\n" +
            "\t\t\"13N200B5SH0\": 155.0\n" +
            "\t},\n" +
            "\t\"phase_end_forward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 132.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 165.0,\n" +
            "\t\t\"13MS20B5SI0\": 148.0,\n" +
            "\t\t\"13MUK0B5SH0\": 115.0,\n" +
            "\t\t\"13N0F0B5SH0\": 165.0,\n" +
            "\t\t\"13N200B5SH0\": 155.0\n" +
            "\t},\n" +
            "\t\"phase_start_backward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 39.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 42.0,\n" +
            "\t\t\"13MS20B5SI0\": 43.0,\n" +
            "\t\t\"13MUK0B5SH0\": 0.0,\n" +
            "\t\t\"13N0F0B5SH0\": 42.0,\n" +
            "\t\t\"13N200B5SH0\": 40.0\n" +
            "\t},\n" +
            "\t\"phase_start_forward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 39.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 42.0,\n" +
            "\t\t\"13MS20B5SI0\": 43.0,\n" +
            "\t\t\"13MUK0B5SH0\": 0.0,\n" +
            "\t\t\"13N0F0B5SH0\": 42.0,\n" +
            "\t\t\"13N200B5SH0\": 40.0\n" +
            "\t},\n" +
            "\t\"speed_backward\": {\n" +
            "\t\t\"13MOD0B5SI0\": -1.0,\n" +
            "\t\t\"13MQJ0B5SI0\": 36.608,\n" +
            "\t\t\"13MS20B5SI0\": 41.257999999999996,\n" +
            "\t\t\"13MUK0B5SH0\": 40.976,\n" +
            "\t\t\"13N0F0B5SH0\": 38.536,\n" +
            "\t\t\"13N200B5SH0\": 42.28\n" +
            "\t},\n" +
            "\t\"speed_forward\": {\n" +
            "\t\t\"13MOD0B5SI0\": 30.315999999999995,\n" +
            "\t\t\"13MQJ0B5SI0\": 23.904,\n" +
            "\t\t\"13MS20B5SI0\": 41.739999999999995,\n" +
            "\t\t\"13MUK0B5SH0\": 27.832000000000004,\n" +
            "\t\t\"13N0F0B5SH0\": 25.536,\n" +
            "\t\t\"13N200B5SH0\": -1.0\n" +
            "\t}\n" +
            "}";
}
